"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const __1 = require("..");
const model_not_initialized_error_1 = require("../../model/shared/model-not-initialized-error");
const base_association_1 = require("../shared/base-association");
class BelongsToManyAssociation extends base_association_1.BaseAssociation {
    constructor(associatedClassGetter, options) {
        super(associatedClassGetter, options);
        this.options = options;
    }
    getAssociation() {
        return __1.Association.BelongsToMany;
    }
    getSequelizeOptions(modelClass, sequelize) {
        const options = Object.assign({}, this.options);
        const associatedClass = this.getAssociatedClass();
        const throughOptions = this.getThroughOptions(modelClass, sequelize);
        options.through = throughOptions;
        options.foreignKey = this.getForeignKeyOptions(modelClass, throughOptions.model, this.options.foreignKey);
        options.otherKey = this.getForeignKeyOptions(associatedClass, throughOptions.model, this.options.otherKey);
        return options;
    }
    getThroughOptions(modelClass, sequelize) {
        const through = this.options.through;
        const model = typeof through === 'object' ? through.model : through;
        const throughOptions = typeof through === 'object' ? Object.assign({}, through) : {};
        if (typeof model === 'function') {
            const throughModelClass = model();
            if (!throughModelClass.isInitialized) {
                throw new model_not_initialized_error_1.ModelNotInitializedError(throughModelClass, {
                    cause: 'before association can be resolved.',
                });
            }
            throughOptions.model = throughModelClass;
        }
        else if (typeof model === 'string') {
            // TODO@robin: Keep using strings instead of generating a through model
            // TODO:       in order to prevent manipulating parameter "sequelize"
            if (!sequelize.throughMap[model]) {
                const throughModel = __1.getThroughModel(model);
                sequelize.addModels([throughModel]);
                sequelize.throughMap[model] = throughModel;
            }
            throughOptions.model = sequelize.throughMap[model];
        }
        else {
            throw new Error(`Through model is missing for belongs to many association on ${modelClass.name}`);
        }
        return throughOptions;
    }
}
exports.BelongsToManyAssociation = BelongsToManyAssociation;
//# sourceMappingURL=belongs-to-many-association.js.map