"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
require("reflect-metadata");
const OriginSequelize = require("sequelize");
const model_service_1 = require("../../model/shared/model-service");
const index_1 = require("../../hooks/index");
const index_2 = require("../../associations/index");
const scope_service_1 = require("../../scopes/shared/scope-service");
const sequelize_service_1 = require("./sequelize-service");
const attribute_service_1 = require("../../model/column/attribute-service");
exports._OriginSequelize = OriginSequelize;
class SequelizeImpl extends exports._OriginSequelize {
    constructor(options) {
        if (typeof options === "string") {
            super(options, sequelize_service_1.prepareOptions({ url: options }));
        }
        else if (sequelize_service_1.hasSequelizeUri(options)) {
            super(options.url, sequelize_service_1.prepareOptions(options));
        }
        else {
            super(sequelize_service_1.prepareOptions(options));
        }
        this.throughMap = {};
        this.models = {};
        if (typeof options !== "string") {
            this.init(options);
        }
    }
    init(options) {
        const sequelizeOptions = options;
        if (sequelizeOptions.models)
            this.addModels(sequelizeOptions.models);
        if (sequelizeOptions.modelPaths)
            this.addModels(sequelizeOptions.modelPaths);
    }
    addModels(arg, modelMatch) {
        const defaultModelMatch = (filename, member) => filename === member;
        const models = model_service_1.getModels(arg, modelMatch || this.options.modelMatch || defaultModelMatch);
        this.defineModels(models);
        this.associateModels(models);
        scope_service_1.resolveScopes(models);
        index_1.installHooks(models);
        models.forEach(model => this.models[model.name] = model);
    }
    /**
     * Processes model associations
     */
    associateModels(models) {
        models.forEach(model => {
            const associations = index_2.getAssociations(model.prototype);
            if (!associations)
                return;
            associations.forEach(association => {
                const options = association.getSequelizeOptions(model, this);
                const associatedClass = association.getAssociatedClass();
                const relation = association.getAssociation();
                model[relation](associatedClass, options);
            });
        });
    }
    /**
     * Creates sequelize models and registers these models
     * in the registry
     */
    defineModels(models) {
        models.forEach(model => {
            const modelName = model_service_1.getModelName(model.prototype);
            const attributes = attribute_service_1.getAttributes(model.prototype);
            const options = model_service_1.getOptions(model.prototype);
            if (!options)
                throw new Error(`@Table annotation is missing on class "${model['name']}"`);
            options['modelName'] = modelName;
            options['sequelize'] = this;
            model['init'](attributes, options);
        });
    }
}
exports.SequelizeImpl = SequelizeImpl;
//# sourceMappingURL=sequelize-impl.js.map